# StarDropToXLS.py - plugin module for StarDrop
#
# Copyright (C) Optibrium Ltd 2011
# stardrop-support@optibrium.com
#
# This module allows a StarDrop data set to be converted into an Excel
# spreadsheet(.xls) file. Optionally, formatting may be applied to rows
# that satisfy some specified criteria

# For advice or feedback, contact stardrop-support@optibrium.com

###########################################################################################
# Acknowledgements
# This code uses the PyExcelerator module developed by Roman V. Kiseliov <roman@kiseliov.ru>

# THIS SOFTWARE IS PROVIDED BY Roman V. Kiseliov ``AS IS'' AND ANY
# EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Roman V. Kiseliov OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
###########################################################################################

import string
import app
import os
import sys
import subprocess
from myCriteria import criteria as criteria
from package_functions import add_site_packages_dir
from DataSetUtils import getDisplayValues

try:
    add_site_packages_dir()
    from xlwt import Font, XFStyle, Pattern, Workbook
except:
    raise RuntimeError("Unable to load xlwt package, check installation.")

# set up styles to use
deffont = Font()
redfont = Font()
redfont.colour_index = 0x02
greenfont = Font()
greenfont.colour_index = 0x03
orangefont = Font()
orangefont.colour_index = 0x35  # this is orange on a default set up of Excel
defstyle = XFStyle()
defstyle.font = deffont
redstyle = XFStyle()
redstyle.font = redfont
greenstyle = XFStyle()
greenstyle.font = greenfont
orangestyle = XFStyle()
orangestyle.font = orangefont


def match(val, c):
    if len(c) == 2:  # simple match
        return val == c[0]
    op = c[0]
    if op == '<':
        return val <= c[1]
    elif op == '>':
        return val > c[1]
    elif op == '<>':
        return val > c[1] and val <= c[2]


def getStyle(val, cr):
    style = defstyle
    for c in cr:
        res = match(val, c)
        if res == True:
            if c[-1] == 0:
                style = greenstyle
            elif c[-1] == 1:
                style = orangestyle
            elif c[-1] == 2:
                style = redstyle

    return style


def save(data, criteria, stats, home):
    ds = app.Dataset()
    ds.parse(data)
    if ds.columnSz() < 1:
        return False
    # print 'saving data set to ' + name + '.XLS'

    # create styles
    headerpattern = Pattern()
    headerpattern.pattern = Pattern.SOLID_PATTERN
    headerpattern.pattern_fore_colour = 0x16
    headerstyle = XFStyle()
    headerstyle.pattern = headerpattern

    # create a worksheet
    wb = Workbook()
    ws = wb.add_sheet("Data")

    # enter the column headers (with or without statistics)
    i = 0
    usestats = {}
    for col in range(0, ds.columnSz()):
        coldata = ds.columnMeta(col)
        ws.write(0, i, ds.columnName(col), headerstyle)
        if stats and coldata[0] in (app.cat, app.cont, app.score, app.regio):
            i = i + 1
            ws.write(0, i, ds.columnName(col) + '_stats', headerstyle)
            usestats[col] = True
        else:
            usestats[col] = False
        i = i + 1

    # enter individual entries, colour coded according to criteria
    for row in range(0, ds.rowSz()):
        i = 0
        for col in range(0, ds.columnSz()):
            style = defstyle
            entry = ds.get(col, row)
            coldata = ds.columnMeta(col)
            try:
                (val, stats) = getDisplayValues(entry, coldata)
            except:
                (val, stats) = ('?', '?')
            header = ds.columnName(col)
            if header in criteria:
                cr = criteria[header]
                style = getStyle(val, cr)
            point = ".0"
            if str(val).endswith(point):
                style.num_format_str = '0'
            else:
                style.num_format_str = '0.00'
            # print val
            ws.write(row + 1, i, val, style)
            if usestats[col] == True:
                i = i + 1
                ws.write(row + 1, i, stats, style)
            i = i + 1

    # save the spreadsheet
    name = home + 'StarDroptoExcel'
    if len(ds.filename) > 0:
        lindex = ds.filename.rfind('/')
        rindex = ds.filename.find('.')
        if lindex > 0 and rindex > lindex:
            name = home + ds.filename[lindex + 1:rindex]
    counter = 1
    origname = name
    while os.path.isfile(name + '.xls'):
        name = origname + '_' + str(counter)
        counter += 1
    wb.save(name + '.xls')
    if sys.platform == "win32":
        os.startfile(name + '.xls')
    else:
        subprocess.call(["open", name + ".xls"])
    return True


def exportToExcel(appui):
    ds = appui.importDS("")
    res = save(ds, criteria, False, os.getcwd())
    if res == False:
        appui.show("Unable to open file in Excel")


def get_stardrop_definitions():
    definitions = [
        {
            'script_name': 'Export to Excel',
            'callback': exportToExcel
        }
    ]

    return definitions
